import os
import sys

__author__ = "Yusuf Can Bayrak <yusufcanbayrak@gmail.com>"
__version__ = "0.1.0-alpha.1"

PYTHON_VERSION = f"{sys.version_info.major}.{sys.version_info.minor}"
PKG_DIR = os.path.dirname(os.path.abspath(__file__))
